package scenari;


import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;

import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.MqttTopic;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.json.JSONException;
import org.json.JSONObject;

import utility.Esecutore;
import utility.Helper;
import utility.SubscribeCallback;

public class Scenari {

	public static boolean shouldPrint;
	private String learnTrigger;
	private String attivaScenari;
	private String luceAntifurto;
	private static ArrayList<String> topicsSub;
	private Automa automa;
	private static String mqttTree;
	public static final String RES_FOLDER = "../res/";
	public static final String CONF_FOLDER = RES_FOLDER+"CONF/";
	private final String FILE_CONF = CONF_FOLDER+"conf.json";
	private final String FILE_ZONA = CONF_FOLDER+"zona.json";

	public MqttClient mqttClient;

	public Scenari(Automa automa) throws JSONException, IOException, MqttException {
		this.automa = automa;

		JSONObject config = new JSONObject(Helper.leggiFile(FILE_CONF));
		String brokerUrl = config.getString("protocol") + "://" + config.getString("broker") + ":" + config.getInt("port");
		String mqttDomain = config.getString("mqttDomain");
		String mqttSubdomain = config.getString("mqttSubdomain");
		mqttTree = mqttDomain + "/" + mqttSubdomain + "/";

		topicsSub = new ArrayList<String>();
		topicsSub.add("to/all"); //mandare la mia descrizione json
		topicsSub.add(getMqttTree("rpc/","scenari")); //mandare lo stato attuale

		// inutile aggiungere i topic, perche subbo gia' tutto gpio/#
		config = new JSONObject(Helper.leggiFile(FILE_ZONA));
		learnTrigger = config.getString("learn-trigger");
		attivaScenari = config.getString("attiva-scenari");
		luceAntifurto = config.getString("luce-antifurto");

		topicsSub.add(getMqttTree("from/","gpio/#"));
		topicsSub.add(getMqttTree("to/","scenari/#"));//{"evento":0,"nome":nome_scenario} per attivare/disattivare uno scenario; risposta su from/tree/scenari/attiva con {"nome":nome_scenario}
		// topicsSub.add(getMqttTree("to/","scenari/salva"));//per rinominare lo scenario piu recente

		String clientId = Long.toString(new Date().getTime()) + "-scenari"; // unique client id
		this.mqttClient = new MqttClient(brokerUrl, clientId, new MemoryPersistence());
		//memory persistence serve per non avere errori in console
		// https://github.com/eclipse/paho.mqtt.java/issues/794
	}

	public void startClient(Esecutore esec) throws MqttException {
		MqttConnectOptions options = new MqttConnectOptions();
		options.setUserName("gruppo2");
		options.setPassword("funziona".toCharArray());
		options.setCleanSession(false);
		mqttClient.setCallback(new SubscribeCallback(this, esec, automa));

		mqttClient.connect(options);

		for(String t: topicsSub)
			mqttClient.subscribe(t);

		// query the antifurto microservice to set the SAME luceAntifurto
		String req = "{\"request\":\"out\"}";
		sendMqttMessage(Scenari.getMqttTree("to", "antifurto/luceAntifurto"), req);
	}

	public void sendMqttMessage(String topic, String msg) throws MqttException {
		final MqttTopic msgTopic = mqttClient.getTopic(topic);
		msgTopic.publish(new MqttMessage(msg.getBytes()));
	}

	public static String getMqttTree(String prefix,String suffix) {
		return (prefix.endsWith("/") ? prefix : prefix+"/") + mqttTree + (suffix.startsWith("/") ? suffix.substring(1) : suffix);
	}

	public static void main(String args[]) throws JSONException, IOException, MqttException {
		if(args.length > 0)
			Scenari.shouldPrint = true;
		Helper.print("scenari started");
		startSystem();
	}


	private static void startSystem() throws JSONException, IOException, MqttException {
		Automa automa = new Automa();
		Scenari scenari = new Scenari(automa);
		Esecutore esec = new Esecutore(automa,scenari);

		scenari.startClient(esec);
		esec.start();
	}

	public String getLearnTrigger() {
		return learnTrigger;
	}

	public String getAttivaScenari() {
		return attivaScenari;
	}

	public void setAttivaScenari(String attivaScenari) throws JSONException, IOException{
		if(!attivaScenari.matches("(IN|in)[0-7]"))
			throw new IllegalArgumentException("attivaScenari non corretto");
		// write attivaScenari to the file zona
		JSONObject config = new JSONObject();
		config = new JSONObject(Helper.leggiFile(FILE_ZONA));
		config.put("attiva-scenari", attivaScenari);
		Helper.scriviFile(config, FILE_ZONA);
		this.attivaScenari = attivaScenari;
	}

	public String getLuceAntifurto() {
		return luceAntifurto;
	}

	public void setLuceAntifurto(String luceAntifurto) throws JSONException, IOException{
		if(!luceAntifurto.matches("(OUT|out)[0-7]"))
			throw new IllegalArgumentException("luceAntifurto non corretta");
		// write luceAntifurto to the file zona
		JSONObject config = new JSONObject();
		config = new JSONObject(Helper.leggiFile(FILE_ZONA));
		config.put("luce-antifurto", luceAntifurto);
		Helper.scriviFile(config, FILE_ZONA);
		this.luceAntifurto = luceAntifurto;
	}
}
