"use strict";

import ApiMqttOut from "./mqtt/api-mqtt-out.js";

const sensoreTemplate = {
    nome: "",
    stato: false,
    utilizzo: "",
};

class Sensori {

    static container = document.getElementById("sensoriContainer");

    static addHereSensori;

    static sensoriBucketIn = [];
    static sensoriBucketOut = [];

    static sensoriDisponibiliIn = [];
    static sensoriDisponibiliOut = [];

    constructor() {
        Sensori.populateSensoriDisponibili();
        Sensori.addHereSensori = Sensori.container.querySelector('ul');
    }
    /**
     * rimuove il sensore dalla lista dei sensori disponibili e lo aggiunge alla lista dei sensori usati; inoltre aggiorna le select
     * @param {String} nome il nome del sensore
     * @returns l'oggetto sensore
     */
    static claimSensore(nome) {
        nome = nome.toUpperCase();
        if (!Sensori.checkNomeSensore(nome))
            return;
        let sensore = null;
        if (nome.startsWith("IN")) {
            const index = Sensori.sensoriDisponibiliIn.findIndex(s => s.nome === nome);
            if (index === -1) {
                // Sensori.liberaSensore(nome);
                console.log('sensore in non trovato')
                throw new Error("Errore impossibile");
            }
            sensore = Sensori.sensoriDisponibiliIn.splice(index, 1)[0];
            Sensori.sensoriBucketIn.push(sensore);
        } else {
            const index = Sensori.sensoriDisponibiliOut.findIndex(s => s.nome === nome);
            if (index === -1) {
                // Sensori.liberaSensore(nome);
                console.log('sensore out non trovato')
                throw new Error("Errore impossibile");
            }
            sensore = Sensori.sensoriDisponibiliOut.splice(index, 1)[0]
            Sensori.sensoriBucketOut.push(sensore);
        }
        Sensori.showAvailableOptions();
        return sensore;
    }

    /**
     * aggiorna le select da cui l'utente puo' scegliere i sensori da usare
     *
     */
    static showAvailableOptions() {
        const selectIn = document.querySelectorAll('.scelta-porta-in');
        for (const select of selectIn)
            Sensori.populateOptions(select, false);
        const selectOut = document.querySelectorAll('.scelta-porta-out');
        for (const select of selectOut)
            Sensori.populateOptions(select, true);
    }
    static populateOptions(selectNode, out = false) {
        const sensori = out ? Sensori.sensoriDisponibiliOut : Sensori.sensoriDisponibiliIn;
        //selectNode.innerHTML = '';
        // remove all options from the selectNode
        while (selectNode.firstChild)
            selectNode.removeChild(selectNode.firstChild);
        const option = document.createElement("option");
        option.value = 'unselected';
        option.innerHTML = 'Scegli una porta di attivazione';
        selectNode.appendChild(option);
        for (const s of sensori) {
            const option = document.createElement("option");
            option.value = s.nome;
            option.innerHTML = s.nome;
            selectNode.appendChild(option);
        }
    }
    /**
     * rimette il sensore nella lista dei sensori disponibili e lo rimuove dalla lista dei sensori usati; inoltre aggiorna le select
     * @param {*} sensore il sensore che non si utilizza piu'
     */
    static liberaSensore(sensore) {
        // the sensor is removed from sensoriDisponibili but isn't yet in sensoriBucket
        if (Sensori.checkNomeSensore(sensore?.nome))
            sensore.nome.startsWith("IN") ? Sensori.sensoriDisponibiliIn.push(sensore) : Sensori.sensoriDisponibiliOut.push(sensore);
        Sensori.showAvailableOptions();
    }

    static showSensore(sensore) {
        // in teoria questo non dovrebbe venire usato ? e solo con quelli out, al massimo

        // sensore.nome = sensore.nome.toLowerCase().trim();
        // if(!Sensori.checkNomeSensore(sensore.nome))
        //     throw new Error("Nome sensore non valido");
        // if(sensore.stato == null || typeof sensore.stato !== "boolean")
        //     throw new Error("Stato sensore non valido");
        // sensore.utilizzo = sensore.utilizzo.trim();
        // if(sensore.utilizzo == null || sensore.utilizzo.length === 0)
        //     throw new Error("Utilizzo sensore non impostato");

        // if(!fromRemove){
        // remove the sensor with the same name from sensoriDisponibili
        // then add the new sensor to sensoriBucket
        // Sensori.claimSensore(sensore.nome);
        // }
        const sensoreEl = Sensori.createSensoreHTML(sensore);
        Sensori.addHereSensori.appendChild(sensoreEl);
        sensoreEl.addEventListener('click', () => {
            ApiMqttOut.testSensore(sensore.nome);//FIXME remove this test
        });
        Sensori.initializaTooltips();
    }

    static createSensoreHTML(sensore) {
        // must set: id="sensore-in0" src="res/led-green.png" alt="sensore in0" class="" title="More info here!"
        const sensoreEl = document.createElement("li");
        sensoreEl.classList.add('d-flex', 'flex-column');
        sensoreEl.innerHTML = `
        <!--<li class="d-flex flex-column">-->
            <span class="d-flex justify-content-center">
                <img id="sensore-${sensore.nome}" src="res/led-${sensore.stato ? 'green' : 'red'}.png" alt="sensore-${sensore.nome}" title="${sensore.utilizzo}" height="70" width="70" data-bs-toggle="tooltip" data-bs-placement="bottom">
            </span>
            <p class="d-flex justify-content-center">${sensore.nome}</p>
        <!--</li>-->
        `;
        return sensoreEl;
    }
    /**
     * creates a new sensor and returns it. It can also be added to the bucket.
     * @param {*} nome
     * @param {*} stato
     * @param {*} utilizzo
     * @param {*} mostraSensore
     * @returns
     */
    static createSensoreDaValori(nome, stato, utilizzo, mostraSensore = false) {
        const sensore = Object.assign({}, sensoreTemplate);
        sensore.nome = nome.toUpperCase().trim();
        sensore.stato = stato ? stato : false;
        sensore.utilizzo = utilizzo ? utilizzo : null;
        if (mostraSensore)
            Sensori.showSensore(sensore);
        return sensore;
    }
    /**
     *
     * @param {String} nome il nome del sensore
     * @returns true se il nome e' corretto ed il sensore e' disponibile
     */
    static checkNomeSensore(nome) {
        if (Sensori.sensoriBucketIn.length + Sensori.sensoriBucketOut.length >= 16 || Sensori.sensoriDisponibiliIn.length + Sensori.sensoriDisponibiliOut === 0)
            return false;
        if (nome == null || nome.length === 0)
            return false;
        if (nome.startsWith("IN")) {
            const number = nome.substring(2);
            if (number == null || isNaN(number) || isNaN(parseInt(number)) || number < 0 || number > 8)
                return false;
            for (const sensore of Sensori.sensoriBucketIn)
                if (sensore.nome === nome)
                    return false;
            return Sensori.sensoriDisponibiliIn.length > 0;
        } else if (nome.startsWith("OUT")) {
            const number = nome.substring(3);
            if (number == null || isNaN(number) || isNaN(parseInt(number)) || number < 0 || number > 8)
                return false;
            for (const sensore of Sensori.sensoriBucketOut)
                if (sensore.nome === nome)
                    return false;
            return Sensori.sensoriDisponibiliOut.length > 0;
        } else return false;
    }

    static populateSensoriDisponibili() {
        for (let i = 0; i < 16; i++) {
            if (i < 8) {
                Sensori.sensoriDisponibiliIn.push(Sensori.createSensoreDaValori("in" + i));
            } else {
                Sensori.sensoriDisponibiliOut.push(Sensori.createSensoreDaValori("out" + (i % 8)));
            }
        }
    }

    static initializaTooltips() {//from bootstrap, mandatory
        const tooltipTriggerList = [].slice.call(document.querySelectorAll('[data-bs-toggle="tooltip"]'));
        const tooltipList = tooltipTriggerList.map(function (tooltipTriggerEl) {
            return new bootstrap.Tooltip(tooltipTriggerEl)
        });
    }
}

export default Sensori;
